%%%Simulation model for Merlin WR200%%%
clear all
clc
close all
%%
%Integration values
sim_time=600;                          %Simulation end time
dt=0.2;                                %Steplength
L=sim_time/dt;                         %Simulation steps
%%                                
%%%%%%%%%%%%%%%%      Dimensions and constants   %%%%%%%%%%%%%%%%%%%%%%%%%%
tau_thruster_max=1382;                  %Max forward thrust [N]
i=1;  
m=3184; %3184                           %Mass of the ROV %3300
g=9.81;                                 %Gravitational acceleration
rho=1024;                               %Density of salt water
V=3.22;  %3.22                          %Volume of WR200
alpha=pi/4;                             %Angle of thrusters
l1=0.73;                                %
l2=0.24;                                %
l3=0.73;                                %
l4=0.84;                                %Length of moment arms
l5=0.1;                                 %
l6=0.84;                                %
r_g=[-0.002341 0.003014 -0.021193];     %Location of CG wrt CO
r_b=[0.12 0 0.197];                     %Location of CB wrt CO
I=[1805+0.05*m 0 -120;0 3050+0.05*m 7;-120 7 2872+0.05*m];   %Inertia matrix
B=rho*g*V;                              %Buoyancy
W=m*g;                                  %Weight
M_RB=1.1*m*eye(3);                      %Mass matrix including added mass
M=[M_RB*eye(3) -m*Smtrx(r_g);m*Smtrx(r_g) I];%System inertia matrix
M_inv=inv(M);
D=diag([1321 2525 2525 192 192 192]);   %Damping matrix
%%%%%%%%%%%%%%%%%%%%   Thruster allocation    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
T=[zeros(1,4) sin(alpha) sin(alpha) -sin(alpha) -sin(alpha);zeros(1,4) cos(alpha) -cos(alpha) -cos(alpha)...
    cos(alpha);ones(1,4) zeros(1,4);l2 -l2 -l2 l2 -l5 l5 -l5 l5;l1 l1 -l3 -l3...
    -l5*0*ones(1,4);zeros(1,4) -l4 l4 -l6 l6];
T_inv=pinv(T);
%%
%%%%%%%%%%%%%%%%%%% LQR settings   %%%%%%%%%%%%%%%%%%%%%%%%%%%
A6=[zeros(6,6) eye(6);zeros(6,6) -M_inv*D];
B6=[zeros(6,6);M_inv];

x_max=0.15; %Max deviation in X
y_max=0.15; %Max deviation in Y
z_max=0.15; %Max deviation in Z
psi_max=5*pi/180; %Max deviation in phi
theta_max=5*pi/180; %Max deviation in theta
phi_max=2*pi/180; %Max deviation in psi

Q6=diag([1/(x_max^2) 1/(y_max^2) 1/(z_max^2) 1/(phi_max^2)...
    1/(theta_max^2) 1/(psi_max^2) 10 10 10 10 10 10]);% Generate Q matrix

% Specify max allowable force in the different DOFs. Chosen rather
% conservative to not break the saturation:
tau_max_x=500; % Max allowable force in X
tau_max_y=500; % Max allowable force in Y
tau_max_z=1000; % Max allowable force in Z
tau_max_psi=100; % Max allowable force in K
tau_max_theta=100; % Max allowable force in M
tau_max_phi=500; % Max allowable force in N

r6=zeros(6,1);

r6(1)=1/(tau_max_x^2);
r6(2)=1/(tau_max_y^2);
r6(3)=1/(tau_max_z^2);
r6(4)=1/(tau_max_phi^2);
r6(5)=1/(tau_max_theta^2);
r6(6)=1/(tau_max_psi^2);

R6=diag(r6); %Make R matrix
[K6,P6,e6]=lqr(A6,B6,Q6,R6); %Generate K, R and eigenvalues
Kd_LQ6=diag(diag(K6(:,7:12))); %Pick out Kd

%%%%%%%%%%%%%%%%% Controller Gains %%%%%%%%%%%%%%%%%%%%%%%%%%%
Kd=diag([Kd_LQ6(1,1) Kd_LQ6(2,2) Kd_LQ6(3,3) 1 1 Kd_LQ6(6,6)]);
LAM=diag([0.1 0.389 0.2 20 1000 0.4]);
Gamma = diag([400 400 400 0]);

Cl=[zeros(6,6) eye(6);-(Kd*LAM)/M -((D/M)+(Kd/M))]; % Closed loop A matrix

%%
%Creating tables for data storage
time=zeros(1,L);                        %Time vector
nu=zeros(6,L);                          %Velocity vector
e_2=zeros(6,L);                        %Error in velocity
e_nu_int=zeros(6,L+1);                  %Error in velocity integrated
tau=zeros(6,L);                         %Control forces and moments                   
eta=zeros(6,L);                         %Position and attitude vector
eta_dot=zeros(6,L);                     %Derivative of position(same as velocities)
e_1=zeros(6,L);                       %Error in position and attitude
e_eta_dot=zeros(6,L);                   %Derivative of error
e_eta_int=zeros(6,L+1);                 %Error in pos. and attitude integrated
eta_corr=zeros(1,L);                    %Vector to store corrections when passing 360 degrees
eta_corr2=zeros(1,L);
nu_dot=zeros(6,L);                      %Derivative of velocities = acceleration
joy_x=zeros(1,L);                       %Joystick input in X
joy_y=zeros(1,L);                       %Joystick input in Y
joy_z=zeros(1,L);                       %Joystick input in Z
joy_n=zeros(1,L);                       %Joystick input in N
nu_ref=zeros(12,L);                     %Reference velocity (filtered desired velocity)
eta_ref=zeros(18,L);                    %Reference pos. and att.(filtered desired pos and att.)
g_eta=zeros(6,L);                       %Restoring forces
a_b=zeros(6,L);                         %Commanded acceleration in body. PD controller for speed
a_n=zeros(6,L);                         %Commanded acceleration in NED. PID controller for pos and att.
tau_thruster=zeros(8,L);                %Force to each thruster
u=zeros(8,L);                           %Frequency to each thruster
u_pros=zeros(8,L);                      %Percentage to each thruster
telapsed=zeros(1,L);                    %Elapsed time
u_corrected=zeros(8,L);                 %Percentage to each thruster, corrected to fit simulator
nu_v = zeros(6,L);
eta_dot_p_d = zeros(6,L);
eta_d_dot_p_d = zeros (6,L);
e_1_dot = zeros(6,L);
nu_v_dot = zeros(6,L);
theta = zeros(14,L);
theta_dot = zeros(14,L);
tau_c_temp = zeros(4,L);
tau_c = zeros(6,L);
nu_relative = zeros(6,L);
eta_d = zeros(6,L);
nu_d = zeros(6,L);
tau_pd = zeros(6,L);
pausetid = zeros(1,L);
X_hat = zeros(2,L);
tau_ned = zeros(6,L);
Y_hat = zeros(2,L);
yaw_hat = zeros(2,L);
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%      UDP connection   %%%%%%%%%%%%%%%%%%%%%%%%%

%Define UDP object
obj1 = instrfind('Type', 'udp', 'RemoteHost', '10.3.0.44', 'RemotePort', 11001, 'Tag', '');

if isempty(obj1)
    obj1 = udp('10.3.0.44', 11001, 'LocalPort',11000);
end

if strcmp(obj1.status,'closed') == true
    fopen(obj1);
end

flushinput(obj1);           %Empty buffer input
flushoutput(obj1);          %Empty buffer output

u_send='0 0 0 0 0 0 0 0';   %Send zeros to all thrusters to initiate contact
tic                         %Start timer
sendt=0;mottatt=0;          %Set sent and recieved counter to zero
%%
%%%%%%%%%%%%%%%%%%%%%%%%  LOOP   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 while telapsed < 1000              %Enter desired time to run in seconds
  
   fwrite(obj1,u_send);             %Write to PLC to be able to recieve
   sendt=sendt+1;                    %Count how many packages sent
   data1 = fscanf(obj1);             %Reading values from PLC
   
   if isempty(data1)==false
       mottatt=mottatt+1;            %Count how many packages recieved.
   end
   data2 = strsplit(data1);             %Split into separate words
   x_c=cell2mat(data2(1,1));            %Convert from cell type to string(char)
   eta(1,i)=str2double(x_c);            %Convert from char to num and place value in x position slot
   y_c=cell2mat(data2(1,2));            %Convert from cell type to string(char)        
   eta(2,i)=str2double(y_c);            %Convert from char to num and place value in y position slot
   z_c=cell2mat(data2(1,3));            %Convert from cell type to string(char)
   eta(3,i)=str2double(z_c);            %Convert from char to num and place value in z position slot
   roll_c=cell2mat(data2(1,5));         %Convert fro cell type to string(char)
   eta(4,i)=str2double(roll_c)*pi/180;  %Convert from char to num and place value in roll attitude slot
   pitch_c=cell2mat(data2(1,6));        %Convert from cell type to strig(char)
   eta(5,i)=str2double(pitch_c)*pi/180; %Convert from char to num and place value in pitch attitude slot
   yaw_c=cell2mat(data2(1,4));          %Convert from cell type to string(char)
   eta(6,i)=str2double(yaw_c)*pi/180;   %Convert from char to num and place value in yaw attitude slot
   
   if i>1                           
       if (eta(6,i)-eta(6,i-1))>pi
           eta_corr(1,i)=eta(6,i)-2*pi;
           eta(6,i)=eta_corr(1,i);      %Discontinuity fix for heading around 360 degrees
       elseif (eta(6,i)-eta(6,i-1))<-pi
           eta_corr2(1,i)=eta(6,i)+2*pi;
           eta(6,i)=eta_corr2(1,i);
       end
   end
    joy_x_c=cell2mat(data2(1,7));        %Convert from cell type to string(char)
    joy_x(i)=str2double(joy_x_c);     %Joystick input in X in -1 to 1
    joy_y_c=cell2mat(data2(1,8));        %Convert from cell type to string(char)
    joy_y(i)=str2double(joy_y_c);     %Joystick input in Y in -1 to 1
    joy_n_c=cell2mat(data2(1,9));        %Convert from cell type to string(char)
    joy_n(i)=str2double(joy_n_c);     %Joystick input in N in -1 to 1
    joy_z_c=cell2mat(data2(1,10));       %Convert from cell type to string(char)
    joy_z(i)=str2double(joy_z_c);     %Joystick input in Z in -1 to 1

    flushinput(obj1);                    %Empty input buffer memory
    flushoutput(obj1);                   %Empty output buffer memory

    %%
    R_mat = [cos(eta(6,i)) sin(eta(6,i)) 0;
             -sin(eta(6,i)) cos(eta(6,i)) 0;  % Rotation matrix! Merk -sin p plass (2,1)
             0 0 1];                          % p grunn av at de bruker North - West - Up  
    P_mat = [R_mat zeros(3,3);
             zeros(3,3) eye(3)];
    if i == 1
       nu(:,1)=0;                                                   
       nu_dot(:,1)=0;
       eta_dot(:,1) = 0;
    else
       eta_dot(:,i)=(eta(:,i)-eta(:,i-1))/dt;                      % Update velocity by taking time derivative         
       nu(:,i)=P_mat\eta_dot(:,i);                                     % of position and transforming it with J to fit body
    end
    eta_d(1:3,i) = eta(1:3,1);
    % eta_d(1,300:end) = eta(1,299)+5;
    eta_d(2,300:end) = eta(2,299)+2;
    eta_d(4:6,i,:) = [0;0;eta(6,1)];%45*(pi/180)
    % eta_d(6,400:end)=eta(6,399)-90*pi/180;
    nu_d(:,i) = [joy_x(i)/100*4.3;joy_y(i)/100*3.2;joy_z(i)/100;0;0;joy_n(i)/100*0.5];%zeros(6,1);%nu(:,i);
    %%
    %%%%%%%%%%%%%%%%%%%%   REFERENCE MODEL   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    omega_ref_nu=diag([2 2 2 1 1 2]);             %Velocity reference model natural frequency
    zeta_ref_nu=1;              %Velocity reference model damping. 1=no overshoot
    omega_ref_eta=diag([0.3 0.3 1 1 1 0.3]);            %Pos and att reference model natural frequency
    zeta_ref_eta=1;             %Pos and att reference model damping 1=no overshoot

    %Velocity reference model
    A_ref_nu=[zeros(6,6) eye(6);-omega_ref_nu^2*eye(6) -2*zeta_ref_nu*omega_ref_nu*eye(6)];    %Reference model system matrix, continuous
    B_ref_nu=[zeros(6,6);omega_ref_nu^2*eye(6)];         %Reference model input matrix, continuous    

    %Position and attitude reference model                Same steps as above
    A_ref_eta=[zeros(6,6) eye(6) zeros(6,6);zeros(6,6) zeros(6,6) eye(6);...
        -omega_ref_eta^3*eye(6) -(2*zeta_ref_eta*eye(6)+eye(6))*omega_ref_eta^2 -(2*zeta_ref_eta*eye(6)+eye(6))*omega_ref_eta];
    B_ref_eta=[zeros(6,6);zeros(6,6);omega_ref_eta^3*eye(6)];

    eta_ref_0=[eta(:,1);zeros(12,1)];
    nu_ref_0=[nu(:,1);zeros(6,1)];

    %Estimator
    A_X_hat =[0 1;0 -1/3.5];
    B_X_hat =[0;0.0005/3.5];
    X_hat_0 = [0 0]';

    A_Y_hat = [0 1;0 -1/2.1];
    B_Y_hat = [0;0.00035/2.1];
    Y_hat_0 = [0 0]';

    A_yaw_hat = [0 1;0 -1/2.2];
    B_yaw_hat = [0;0.000375/2.2];
    Z_yaw_0 = [0 0]';

    if i <= 16
        X_hat(:,i) = (eye(2)+dt.*A_X_hat)*X_hat(:,i) + dt*B_X_hat*tau(1,i);
    else
        X_hat(:,i) = (eye(2)+dt.*A_X_hat)*X_hat(:,i-1) + dt*B_X_hat*tau(1,i-16);
    end

    if i<=30
        Y_hat(:,i) = (eye(2)+dt.*A_Y_hat)*Y_hat(:,i) + dt*B_Y_hat*tau(2,i);
    else
        Y_hat(:,i) = (eye(2)+dt.*A_Y_hat)*Y_hat(:,i-1) + dt*B_Y_hat*tau(2,i-30);
    end
    if i<=40
        yaw_hat(:,i) = (eye(2)+dt.*A_yaw_hat)*yaw_hat(:,i) + dt*B_yaw_hat*tau(6,i);
    else
        yaw_hat(:,i) = (eye(2)+dt.*A_yaw_hat)*yaw_hat(:,i-1) + dt*B_yaw_hat*tau(6,i-40);
    end    
    if i==1 % First iteration, use initial state as nu_ref_0 and eta_ref_0
        nu_ref(:,i)=(eye(12)+dt.*A_ref_nu)*nu_ref_0+dt*B_ref_nu*nu_d(1:6,i);    %Velocity 
        eta_ref(:,i)=(eye(18)+dt.*A_ref_eta)*eta_ref_0+dt*B_ref_eta*eta_d(1:6,i); %Pos and att.
    %     X_hat(:,i) = (eye(2)+dt.*A_X_hat)*X_hat_0 + dt*B_X_hat*tau(1,i);
    else    % Then feed the reference signal through the filter and get a smooth reference with derivatives and second derivatives(eta only)
        nu_ref(:,i)=(eye(12)+dt.*A_ref_nu)*nu_ref(:,i-1)+dt*B_ref_nu*nu_d(1:6,i);%Velocity
    %     X_hat(:,i) = (eye(2)+dt.*A_X_hat)*X_hat(:,i-1) + dt*B_X_hat*tau(1,i-1);
        eta_ref(:,i)=(eye(18)+dt.*A_ref_eta)*eta_ref(:,i-1)+dt*B_ref_eta*eta_d(1:6,i);%Pos and att.
    end
    %%
    %%%%%%%%%%%%%%%%%      CONTROL LOOP   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    g_eta(:,i)=[0 0 0 -4820*cos(eta(5,i))*sin(eta(4,i))...          %
            -4820*sin(eta(5,i))*cos(eta(5,i))*cos(eta(4,i)) 0];         %Update restoring forces vector
    e_1(:,i) = (eta(:,i)-eta_ref(1:6,i));
    if e_1(6,i)<=-pi                                              %Discontinuity fix for heading
        e_1(6,i)=2*pi-e_1(6,i);
    elseif e_1(6,i)>pi
        e_1(6,i)=-2*pi+e_1(6,i);
    end

    eta_dot_p_d(:,i) = 0.8*eta_ref(7:12,i);
    eta_d_dot_p_d(:,i) = 0.1*eta_ref(13:18,i);
    e_1_dot(:,i) = nu(:,i) - eta_dot_p_d(:,i);
    nu_v(:,i) = -LAM*e_1(:,i)+eta_dot_p_d(:,i);
    nu_v_dot(:,i) = -LAM*e_1_dot(:,i)+eta_d_dot_p_d(:,i);
    e_2(:,i)=eta_dot(:,i)-nu_v(1:6,i);

    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%% Generate regressor          %%%%%%%%%%%%%%%%%%%%%%%%%%

    phi = [nu_v_dot(1,i) 0 0 0 -abs(nu(1,i))*nu(1,i) 0 0 0 0 0 0 cos(eta(6,i)) -sin(eta(6,i)) 0;
           0 nu_v_dot(2,i) 0 0 0 -abs(nu(2,i))*nu(2,i) 0 0 0 0 0 sin(eta(6,i)) cos(eta(6,i)) 0;
           0 0 nu_v_dot(3,i) 0 0 0 -abs(nu(3,i))*nu(3,i) 0 0 0 0 0 0 1;
           0 0 0 nu_v_dot(6,i) 0 0 0 -abs(nu(6,i))*nu(6,i) 0 0 0 0 0 0];
    %%

    %%%%%%%%%%%%%%%%%%%%%%%%   Adaptation    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    tau_c_temp(:,i) = phi*theta(:,i);
    tau_c(:,i) = [tau_c_temp(1,i) tau_c_temp(2,i) theta(14,i) 0 0 tau_c_temp(4,i)]';

    %     tau_c(:,i) = [0 0 theta(14,i) 0 0 0]';
    theta_dot(:,i) = phi'*Gamma*e_2([1 2 3 6],i);
    theta(:,i+1) = theta(:,i)+dt*theta_dot(:,i);

    %%%%%%%%%%%%%%%%%%%   Set output   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    tau_pd(:,i) = -e_1(:,i) - Kd*e_2(:,i);
    tau_ned(:,i) = tau_pd(:,i)-tau_c(:,i);
    tau(:,i) = P_mat\tau_ned(:,i);%+g_eta(:,i);

    tau_thruster(:,i)=pinv(T)*tau(:,i);                                 %Desired force from each thruster in [N] and [Nm]  
                                                                        %Desired force is unlimited, but the actual force
                                                                            %must be limited
    if i>1
        for s=1:8                                                           %Check to see if thruster is at limit
            if abs(tau_thruster(s,i))>=tau_thruster_max
                tau_thruster(s,i)=(tau_thruster(s,i)/abs(tau_thruster(s,i)))*tau_thruster_max;                  %Set thruster to limit
            end
        end
    end

    u(:,i)=83*sin(0.0007909*tau_thruster(:,i));

    u_corrected(1,i)=u(6,i);
    u_corrected(2,i)=u(5,i);
    u_corrected(3,i)=u(7,i);
    u_corrected(4,i)=u(8,i);  %Changes numbering of thrusters to fit simulator
    u_corrected(5,i)=u(2,i);
    u_corrected(6,i)=u(1,i);
    u_corrected(7,i)=u(3,i);
    u_corrected(8,i)=u(4,i);

    u_pros(:,i)=(u_corrected(:,i)/100)*100;                %Caluclates how many percent the thrusters should be at

    u_send=sprintf('%f ',u_pros(:,i));                  %Stores the percentages in u_send as a string.

    telapsed(i)=toc;                                    %Updates elapsed time
    
%%%%%%%%%%%%%%%%%%%%%%%%%  REAL TIME PLOTS      %%%%%%%%%%%%%%%%%%%%%%%%%%%

%    plot(telapsed(1:i),joy_x(1,1:i));hold on;
%    plot(telapsed(1:i),nu(1,1:i),'r'); 
%    plot(telapsed(1,1:i),joy_x(1:i),'b');hold on;
% plot(telapsed(1,1:i),eta_ref(1,1:i),'r');hold on;

figure(1)
subplot(3,2,1)
plot(telapsed(1,1:i),eta(1,1:i)),title('N');hold on;
plot(telapsed(1,1:i),eta_ref(1,1:i));
subplot(3,2,3)
plot(telapsed(1,1:i),eta(2,1:i)),title('E');hold on;
plot(telapsed(1,1:i),eta_ref(2,1:i));
subplot(3,2,5)
plot(telapsed(1,1:i),eta(3,1:i)),title('D');hold on;
plot(telapsed(1,1:i),eta_ref(3,1:i));
subplot(3,2,2)
plot(telapsed(1,1:i),eta(4,1:i)),title('\phi');hold on;
plot(telapsed(1,1:i),eta_ref(4,1:i));
subplot(3,2,4)
plot(telapsed(1,1:i),eta(5,1:i)),title('\theta');hold on;
plot(telapsed(1,1:i),eta_ref(5,1:i));
subplot(3,2,6)
plot(telapsed(1,1:i),eta(6,1:i)*180/pi),title('\psi');hold on;
plot(telapsed(1,1:i),eta_ref(6,1:i)*180/pi);

figure(2)
subplot(3,2,1)
plot(telapsed(1,1:i),nu(1,1:i)),title('u');hold on
plot(telapsed(1,1:i),X_hat(2,1:i));
subplot(3,2,3)
plot(telapsed(1,1:i),nu(2,1:i)),title('v');hold on
plot(telapsed(1,1:i),Y_hat(2,1:i));
subplot(3,2,5)
plot(telapsed(1,1:i),nu(3,1:i)),title('w');
subplot(3,2,2)
plot(telapsed(1,1:i),nu(4,1:i)),title('p');
subplot(3,2,4)
plot(telapsed(1,1:i),nu(5,1:i)),title('q');
subplot(3,2,6)
plot(telapsed(1,1:i),nu(6,1:i)),title('r');hold on
plot(telapsed(1,1:i),yaw_hat(2,1:i));
% figure(2)
% plot(telapsed(1,1:i),nu_ref(1,1:i),'b');hold on;
% plot(telapsed(1,1:i),nu_d(1,1:i),'red');
% figure(3)
% plot(telapsed(1,1:i),nu(1,1:i),'red');

% figure(3)
% plot(telapsed(1:i),tau(6,1:i),'c');
%    plot(eta(2,1:i),eta(1,1:i),'bo');hold on;
%    plot(eta_d(2,1:i),eta_d(1,1:i),'bo')
   %
   drawnow
   pausetid(i)=dt-(toc-telapsed(i));
   pause(pausetid)                         %Pauses for sampling time
    
   i=i+1;                                              %Update count
         
 end
%%
%%%%%%%%%%%%%%%%%%%%%%   Plots   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%close all
% figure(1)
% subplot(3,1,1)
% plot(t(1:L),nu(1,:)),title('Surge velocity');grid;hold on;
% plot(t(1,1:L),nu_d(1,:),'red');
% subplot(3,1,2)
% plot(t(1:L),nu(2,:)),title('Sway velocity');grid;hold on;
% plot(t(1,1:L),nu_d(2,:),'red');
% subplot(3,1,3)
% plot(t(1:L),nu(3,:)),title('Heave velocity');grid;hold on;
% plot(t(1,1:L),nu_d(3,:),'red');

%  figure(2)
%  plot(telapsed(1:i),tau(1,1:i));grid
% subplot(2,1,1)
% plot(telapsed(1:i),tau(1,1:i)),title('Input in surge');grid;
% % subplot(2,1,2)
% plot(telapsed(1:i),tau(2,1:i)),title('Input in sway');grid;
% subplot(3,1,3)
% plot(telapsed(1:i),v_current(1,1:i)),title('Current velocity');grid;
% subplot(4,1,4)
% plot(telapsed(1:i),e_nu(1,1:i)),title('Surge error');grid;

% figure(3)
% scatter3(eta(1,:),eta(2,:),-eta(3,:)),title('Position N,E,D');hold on
% plot3(eta_d(1,:),eta_d(2,:),-eta_d(3,:),'ro');
% 
% figure(4)
% subplot(3,1,1)
% plot(t(1,1:L),eta(5,1:end)),title('Position N');grid;hold on
% plot(t(1,1:L),eta_d(5,1:end),'red');
% subplot(3,1,2)
% plot(t(1,1:L),eta(2,1:end)),title('Position E');grid;hold on
% plot(t(1,1:L),eta_d(2,1:end),'red');
% subplot(3,1,3)
% plot(t(1,1:L),eta(3,1:end)),title('Position D');grid;hold on
% plot(t(1,1:L),eta_d(3,1:end),'red');


% figure(5)
% imagesc([0 500],[0 500],flipdim(img,1));hold on;
% set(gca,'ydir','normal');title('Position N, E');
% plot(fliplr(eta(1,1:i-1)),eta(2,1:i-1),'g','linewidth',2);grid;
% plot(eta_r(1,:),eta_r(2,:),'r','linewidth',2);
% plot(wpt.pos.x,wpt.pos.y,'go');

% plot(eta_d(1,100),eta_d(2,100),'ro');
% 
% figure(6)
% subplot(3,1,1)
% plot(telapsed(1:i),e_eta(1,1:i)),title('Error N position');grid
% subplot(3,1,2)
% plot(telapsed(1:i),e_eta(2,1:i)),title('Error E position');grid
% subplot(3,1,3)
% plot(telapsed(1:i),e_eta(3,1:i)),title('Error D position');grid
 
% figure(7)
% subplot(3,1,1)
% plot(eta(4,1:L),eta(6,1:L)),title('Attitude theta,phi');grid;hold on
% plot(eta_d(4,L),eta_d(6,L),'ro');
% subplot(3,1,2)
% plot(time,eta(5,:)),title('Attitude theta');grid;hold on
% plot(time(1:L),eta_r(5,:),'r');
% subplot(3,1,3)
% plot(time,eta(6,:)),title('Attitude psi');grid;hold on
% plot(time(1,1:L),eta_r(6,:),'r');

% figure(8)
% trajectory2(eta(1,1:L),eta(2,1:L),-eta(3,1:L),eta(5,1:L),eta(4,1:L),eta(6,1:L),1,L/50,'helicopter')

% figure(9)
% stem(time(1:L)',AntiWU(1,:)');grid

% figure(10)
% plot3(wpt.pos.x(:),wpt.pos.y(:),-wpt.pos.z(:),'o');hold on;
% plot3(x_p,y_p,-z_p);grid;

% figure(11)
% plot(telapsed(1:end),joy_x(1:i));hold on;
% plot(telapsed(1:end),nu_r(1,1:i));grid;
%%