clear all
close all


%%%%%%%%%%%%%%%%      Dimensions and constants   %%%%%%%%%%%%%%%%%%%%%%%%%%
m=3184; %3184                           %Mass of the ROV %3300
g=9.81;                                 %Gravitational acceleration
rho=1024;                               %Density of salt water
V=3.22;  %3.22                          %Volume of WR200
alpha=pi/4;                             %Angle of thrusters
l1=0.73;                                %
l2=0.24;                                %
l3=0.73;                                %
l4=0.84;                                %Length of moment arms
l5=0.1;                                 %
l6=0.84;                                %
r_g=[-0.002341 0.003014 -0.021193];     %Location of CG wrt CO
r_b=[-0.029 0.005 -0.158];              %Location of CB wrt CO
I=[1819+0.05*m 0 -120;0 3064+0.05*m 7;-120 7 2887+0.05*m];   %Inertia matrix
B=rho*g*V;                              %Buoyancy
W=m*g;                                  %Weight
M_RB=1.1*m*eye(3);                      %Mass matrix including added mass
M_A1 = m*0.1*eye(3);
M_A2 = m*0.05*eye(3);
M=[M_RB*eye(3) -m*Smtrx(r_g);m*Smtrx(r_g) I];%System inertia matrix
M_inv=inv(M);
D=diag([1321 2525 2525 192 192 192]);   %Damping matrix

%%%%%%%%%%%%%%%%%%%%   Robustness check       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
m_hat = m;
I_hat = [1819+0.05*m_hat 0 -120;0 3064+0.05*m_hat 7;-120 7 2887+0.05*m_hat];
M_RB_hat = 1.1*m_hat*eye(3);
M_hat = [M_RB_hat*eye(3) -m_hat*Smtrx(r_g);m_hat*Smtrx(r_g) I_hat];
M_hat_inv = inv(M_hat);
D_hat = D;

%%%%%%%%%%%%%%%%%%%%   Thruster allocation    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
T=[zeros(1,4) sin(alpha) sin(alpha) sin(alpha) ...
    sin(alpha);zeros(1,4) -cos(alpha) cos(alpha) -cos(alpha)...
    cos(alpha);ones(1,4) zeros(1,4);l2 -l2 -l2 l2 ...
    -l5 l5 -l5 l5;-l1 -l1 l3 l3 -l5*ones(1,4);zeros(1,4) -l4 l4 l6 -l6];
T_inv=pinv(T);
%%%%%%%%%%%%%%%%   Simulation settings    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
stepsize=0.01;
h=0.1;
Tend=50;
time=0;
eta=zeros(6,1);
eta_desired=zeros(6,1);
%%%%%%%%%%%%%%%%%%%%%% Environment settings    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
current_angle = 45;
current_velocity =0.5;
in_rad = current_angle*(pi/180);
a=in_rad;
b=current_velocity;

current=[b*cos(a) b*sin(a) 0 0 0 0]'; % Current in NED frame

%%%%%%%%%%%%%%%%%%%%%   Initial and desired values   %%%%%%%%%%%%%%%%%%%%%%
eta0=[0 0 -10 0 0 0*(pi/180)]';
eta_d=[0 0 -10 0 0 0*(pi/180)]';
nu_d=[0 0 0 0 0 0]';
nu0=[0 0 0 0 0 0]';

%%%%%%%%%%%%%%%%%%% 2 LQR controller settings   %%%%%%%%%%%%%%%%%%%%%%%%%%%
A6=[zeros(6,6) eye(6);zeros(6,6) -M_hat_inv*D_hat];
B6=[zeros(6,6);M_hat_inv];

x_max=0.15; %Max deviation in X
y_max=0.15; %Max deviation in Y
z_max=0.5; %Max deviation in Z
psi_max=2*pi/180; %Max deviation in phi
theta_max=2*pi/180; %Max deviation in theta
phi_max=2*pi/180; %Max deviation in psi

Q6=diag([1/(x_max^2) 1/(y_max^2) 1/(z_max^2) 1/(phi_max^2)...
    1/(theta_max^2) 1/(psi_max^2) 10 10 10 10 10 10]);% Generate Q matrix

% Specify max allowable force in the different DOFs. Chosen rather
% conservative to not break the saturation:
tau_max_x=1000; % Max allowable force in X
tau_max_y=1000; % Max allowable force in Y
tau_max_z=2000; % Max allowable force in Z
tau_max_psi=1000; % Max allowable force in K
tau_max_theta=1000; % Max allowable force in M
tau_max_phi=1000; % Max allowable force in N

r6=zeros(6,1);

r6(1)=1/(tau_max_x^2);
r6(2)=1/(tau_max_y^2);
r6(3)=1/(tau_max_z^2);
r6(4)=1/(tau_max_phi^2);
r6(5)=1/(tau_max_theta^2);
r6(6)=1/(tau_max_psi^2);

R6=diag(r6); %Make R matrix

[K6,P6,e6]=lqr(A6,B6,Q6,R6); %Generate K, R and eigenvalues

Kp_LQ6=diag(diag(K6(:,1:6))); %Pick out Kp
Kd_LQ6=diag(diag(K6(:,7:12))); %Pick out Kd
Ki_LQ6=diag([500 280 650 15000 15000 15000]);

Acl6=(A6-B6*K6); %Closed loop A matrix
% Acl6_bunn_v=diag(diag(Acl6_temp(7:12,1:6)));
% Acl6_bunn_h=diag(diag(Acl6_temp(7:12,7:12)));

eig_Acl6=eig(Acl6); %Eigenvalues of closed loop system

Kp_PID = Kp_LQ6;
Kd_PID = Kd_LQ6;
Ki_PID = Ki_LQ6;

%%%%%%%%%%%%%%%%% 3 Adaptive PD + compensation  %%%%%%%%%%%%%%%%%%%%%%%%%%%
K_Ad=0*diag([1 1 1 1 1 1]);
K2=Kd_LQ6;%diag(diag(-(Acl6(7:12,7:12)*M+D)));
K1=K2\Kp_LQ6;%diag(diag(-(Acl6(7:12,1:6)/Kd_Ad)*M));
K_theta=diag([0.00005 0.0001 0.0005 0.0001 0.0001 0.00002]);%0.002*eye(6);%0.0012
K_theta_inv=inv(K_theta);
Gamma = diag([20000 20000 20000 50000]);

AclAnt=[zeros(6,6) eye(6);-(K2*K1)/M -((D/M)+(K2/M))];